c   fortran90 by s.izumi 2000-05-0 8
      module fem_arrays
      implicit integer (i-n)
      implicit double precision (a-h,o-z)
c  idm   : dimension
c  inmte : max. number of a node in a element --> 8
c  inmnm : max. size of element stiffness matrix  inmte*idim
c  ird   : max. number of integration point in a element
c  icom  : number of stress component (2-dim->4 ,3dim->6)
      parameter(idim=3,inmte=20,inmnm=inmte*idim,icom=6,nfile=10)
c--------------------------------------------------------------
c for subroutie data
c  ngauss : order of gauss integration
c  npara  : analysis condition (0:plain stress,1:plain strain)
cccc  t      : thickness
c  ntn    : total number of nodes
c  nte    : total number of elements
c  ntf    : total number of freedoms
cccc  ie     : element number(variables)
c  ntfor  : total number of freedoms with initial load
c  ntdis  : total number of freedoms with initial displacement
c  nna    : the total number of integration points
c  mnum   : number of the kind of material
c  nb     : half band width
c--------------------------------------------------------------
        character*70 fnum(nfile)
        common /ch1/ fnum
c
	common /nfm1/ ngauss,npara,ntn,nte,ntf,nna,mnum
	common /nfm2/ ntfor,ntdis,nb
	common /nfm3/ ie,na
cccc for iccg method
	common /nfm4/ nsyk
        common /nfm6/ MITR,NITR,ISEL
cccc for allocate
        common /nfm5/ nrd,nnmte,nnmnm
c
cccc for equation
c  naeq : number of equation
c     neqb : root freedom
c     nteq : number of copy freedom
c     neq  : copy freedom
	common /eq0/ naeq
	common /eq1/ neqb(5),nteq(5)
	common /eq2/ neq(5,100)
c
c  D(i,j) : elastic matrix (1-x,2-y,3-z,4-xy,5-xz,6-yz)
        common /dm1/ d(icom,icom)
        common /dm2/ xxi,eta,zeta,hnum
c----------------------------------------------------------------------
c  N    : shape function Ni
c  JINV : Inverse of Jacobian matrix
c  JM   : Jacobian matrix
c  DETJ   : det[J]
c----------------------------------------------------------------------
        double precision jinv(idim,idim),jm(idim,idim)
        common /dm3/ jinv,jm
        common /dm4/ detj
cccc for iccg method
        common /dm5/ FCON,DCON,EPS,ERR,CONST
c---------------------------------------------------------------------
c  NMTE(i) : number of a node in i-th element
c  NN(i,j) : node number of i-th local node of element j 
c  IFOR(i) : freedom number of i-th initial load point
c  IDIS(i) : freedom number of i-th initial disp. point
c  NGNG(i) : number of integration point in i-th element
c  NOM(i)  : material number of i-th element
c  NKINF(i) : kind of element 1--> infinite element
c  IECON(i)  : ELASTIC CHARACTER of i-th material
c  NEUS(i) : i : PROGRAM NODE NUMBER , NEUS(I) : USER ELEMENT NUMBER 
c  NNUS(i) : i : PROGRAM NODE NUMBER , NNUS(I) : USER NODE NUMBER 
c  NSGAV(i) :  summation number of global node i
      allocatable :: nmte(:),nn(:,:),ngng(:),nom(:),nkinf(:),neus(:)
      allocatable :: ifor(:),idis(:),iecon(:)
      allocatable :: nnus(:)
      allocatable :: nf(:)
      allocatable :: nsgav(:)

c  COORD(i,j) : i-th coordinate of node j 
c  ECON(i,j) : i-th elastic constants of j-th material 
c  FB(i)   : load value of i-th initial load point
c  UB(i)   : disp. value of i-th initial disp. point
c  EK(i,j) : element stiffness matrix 
c  GK(i,j) : global stiffness matrix  i: matrix size  j:half band width
c  F(i)    : force vector of node i
c  U(i)    : displacement value of i-th freedom
c  AXI(i)  : x-coordinate of i-th integration point ( -1 < AXI  < 1 )
c  AETA(i) : y-coordinate of i-th integration point ( -1 < AETA < 1 )
c  AZETA(i): z-coordinate of i-th integration point ( -1 < AZETA < 1 )
c  H(i)   : weight parameter of i-th integration point
c  DNDXI(i): dNi/dXI, DNDETA(i) : dNi/dETA
c  DXIF(i): dNi/dXI, DETIF(i) : dNi/dETA  : FOR INFINITE ELEMENT
c  B(i,j)  : elemental stiffness matrix
c  BM(i)   : For strain (B matrix for i-th integration point)
c  BT(i)   : transposed B(6,INMNM) matrix 
c  BTD(i)  : BT(INMNM,ICOM)*D(ICOM,ICOM)
c  C(i,j)  : BT(INMNM,ICOM)*D(ICOM,ICOM)*B(ICOM,INMNM) Bt*D*B
c  EP(i,j,k) : i-th component strain at j-th element
c                k-th local integration point
c  SG(i,j,k) : i-th component stress at j-th element
c                k-th local integration point
c  SGN(J,I) :  j:component i:node number(global)
c----------------------------------------------------------------------
c
      allocatable :: coord(:,:),econ(:,:)
      allocatable :: fb(:),ub(:)
      allocatable :: ek(:,:),gk(:,:),f(:),u(:)
      allocatable :: axi(:),aeta(:),azeta(:),h(:)
      allocatable :: dndxi(:),dndeta(:),dndzeta(:)
      allocatable :: dxif(:),detif(:),dzeif(:)
      allocatable :: b(:,:),bm(:,:,:)
      allocatable :: bt(:,:),btd(:,:),c(:,:)
      allocatable :: ep(:,:,:),sg(:,:,:)
      allocatable :: sgn(:,:),epn(:,:)
c  shn    : shape function Ni
c  shnif  : shape function Ni for infinite element
      allocatable :: shn(:)
      allocatable :: shnif(:)
c
ccc FOR ICCG METHOD
c NSMALL(I) : see data.f
c INDEX2,INDEX5,NSUM --> gyouretsu keisan sofrware p318
      allocatable :: nsmall(:),nsnode(:,:)
      allocatable :: index2(:),index5(:),nsum(:)
      allocatable :: syk(:)
      allocatable :: diag(:),dd(:),t1(:),t2(:),t3(:),t4(:)
      allocatable :: iwk(:)
ccc FOR CHOLESKY METHOD
c      DIMENSION GKC(IBN,IGK),DR(IGK),ZM(IGK)
c
      end module fem_arrays
c---------------------------------------------------------------------
      module work_arrays
      implicit integer (i-n)
      implicit double precision (a-h,o-z)
      parameter(im=6000,nmlt=2,np8=20,ncut=20,ifile=25)
      parameter(im5=im/nmlt,zplot=3.0)
ccccc np8 must be equal to ncut !!!
c      parameter(wg_si=1.0d0,wg_h=0.036d0,wg_c=0.42892)
c physical constants
      data dcon/5.4309d0/
      data ev,boltz,wsi /1.60219D-19,1.38062D-23,4.6643445D-26/
c atomic mass
      data wg_si,wg_h,wg_c /1.0d0,0.036D0,0.42892d0/
      character*70 f_number(ifile)
c----------------------------------------------------------------
      common /ch2/ f_number
c
c-------------------------------------------
c     dt   : time step (kizami) ( constant )
c     detg : det(G)
c     vol  : present volume
c     vol0 : initial volume    ( constant )
ccc for npt ensemble
c     wm   : weight of wall    ( constant )
c     pex  : external pressure
ccc
c     cutdeg  : degree of cutoff ( constant )
c     nstep   : present MD step
c     ntsteps : total MD step ( constant )
c     navestep : interval step of averaging ( constant )
c     nbstep   : interval step of book-keeping ( constant )
c     ntemp    : temperature control
c     nsi     : number of silicon  atom     
c     ncarb   : number of carbon   atom
c     nhydr   : number of hydrogen atom
c-------------------------------------------
cccc new common sentense
      common /pmd1/ xlmax,xlmin,ylmax,ylmin,zlmax,zlmin
      common /pmd2/ dt,tff,wm,pex,cutdeg,bc_si,tf_sub
      common /pmd3/ hmat(3,3),hmat0(3,3),hmat0i(3,3),hmatt0(3,3),
     & hmatt0i(3,3),smat(3,3),hmati(3,3),hmatt(3,3),hmatti(3,3),
     & gmat(3,3),gmati(3,3),sigma(3,3),apst0(3,3),vol0,vol
      common /pmd4/ tnw(6),uk(6),tnw0(6),uk0(6),vij_all
c
      common /nmd0/ isub1,iden1,isub2,iden2,isub3,iden3,
     & iall,icls,isub,itmp,ifix,nsi,nhydr,ncarbc
      common /nmd1/ ntsteps,ntemp,nstat,nbstep,nout_out,nout_mon
     &,nout_nc,nstep,navestep
      common /nmd2/ ndivxn,ndivyn,ndivzn,nbrmax
c
c Constant variable dependent on a atomic element 
c-----------------------------------------------------------------
ccc Si-H system s->Si h->H (pairwise components)
      common /con1/ a1_ss,a1_sh,a1_hh,b2_ss,b2_sh,b2_hh
     &             ,amd1_ss,amd1_sh,amd1_hh
     &             ,amd2_ss,amd2_sh,amd2_hh
      common /con2/ cutr_ss,cutd_ss,cutr_sh,cutd_sh,cutr_hh,cutd_hh
     &             ,req_ss,req_sh,req_hh
ccc Si-H system s->Si h->H ( \eta=enu_?? , \delta=enu2_?? )
      common /con3/ enu_ss,enu_sh,enu_hh,enu2_ss,enu2_sh,enu2_hh
ccc Si-H system s->Si h->H ( bond angle function Type2)
      common /con4/ gc_ss,gc_sh1,gc_sh2,gc_hh,gd_ss,gd_sh1,gd_sh2
     &             ,gd_hh
c* h--> H(N)[Si(Murty)] , h-->-1.0[H(Murty)]   : fspgh.f
ccc Si-H system s->Si h->H ( bond length terms )
      common /con5/ alp_ss,alp_sh1,alp_sh2,alp_hh
     &             ,bet_ss,bet_sh1,bet_sh2,bet_hh
c* alp_sh2=bet_sh2=0.0 --> NONE
c
ccc Si-C system s->Si c->C (pairwise components)
      common /con11/ a1_cc,a1_cs,b2_cc,b2_cs
     &              ,amd1_cc,amd1_cs,amd2_cc,amd2_cs
      common /con12/ cutr_cc,cutd_cc,cutr_cs,cutd_cs
     &              ,req_cc,req_cs
ccc Si-C system s->Si c->C ( \eta=enu_?? , \delta=enu2_?? )
      common /con13/ enu_cc,enu2_cc
ccc Si-C system s->Si c->C ( bond angle function Type1)
      common /con14/ qdd_ss,qdd_cc, qh_ss, qh_cc
     &              ,qcc_ss,qcc_cc,qcd_ss,qcd_cc
     &              , qa_ss, qa_cc
ccc Si-C system s->Si c->C ( bond length terms )
      common /con15/ alp_cc,alp_cs,bet_cc,bet_cs
c* alp_cc=bet_cc=0.0 --> NONE , alp_cs=bet_cs=0.0 --> NONE 
c
ccc  C-H system c->C  h->H (pairwise components)
      common /con21/ a1_cc2,a1_ch,b2_cc2,b2_ch
     &              ,amd1_cc2,amd1_ch,amd2_cc2,amd2_ch
      common /con22/ cutr_cc2,cutd_cc2,cutr_ch,cutd_ch
     &              ,req_cc2,req_ch
ccc  C-H system c->C  h->H ( \eta=enu_?? , \delta=enu2_?? )
      common /con23/ enu_cc2,enu2_cc2
ccc  C-H system c->C  h->H ( bond angle function Type1)
      common /con24/ qdd_cc2, qh_cc2
     &              ,qcc_cc2,qcd_cc2,qa_cc2
ccc  C-H system c->C  h->H ( bond length terms )
      common /con25/ alp_cc2,bet_cc2
ccc for cutoff
      common /bc/ bcutss,bcutsh,bcuthh
     &           ,bcutcc,bcutcc2,bcutcs,bcutch
      common /bg/ bug
c----------------------------------------------------------------
c atomic coordinates
      allocatable ::  x(:),y(:),z(:)
c atomic velocity
      allocatable :: vx(:),vy(:),vz(:)
c atomic force
      allocatable :: fx(:),fy(:),fz(:)
c image coordinate
      allocatable :: xi(:),yi(:),zi(:)
c image velocity
      allocatable :: vxi(:),vyi(:),vzi(:)
c stress and shape tensor
      allocatable :: stress(:,:)
c atomic potential & kinetic energy 
      allocatable :: vij(:),ekin(:)
      allocatable :: ddx2(:),ddy2(:),ddz2(:)
      allocatable :: dx(:),dy(:),dz(:)
c for tersoff_h.f
      allocatable :: r(:),xr(:),yr(:),zr(:),
     &          f1(:),f1d(:),f1dd(:),af(:),
     &          bf(:),daf(:),dbf(:), elam2(:)
      allocatable :: fnij(:,:)
c for book-keeping
      allocatable ::  rr(:),rtest(:)
      allocatable ::  rx0(:),rx1(:),rx2(:),
     &          sx0(:),sx1(:),sx2(:),
     &          ry0(:),ry1(:),ry2(:),
     &          sy0(:),sy1(:),sy2(:),
     &          rz0(:),rz1(:),rz2(:),
     &          sz0(:),sz1(:),sz2(:),
     &          rxi(:),ryi(:),rzi(:),rbook(:)
c for averaging
      allocatable :: avr_sc(:),avr_at(:,:)
c
Cr----- these are for array for vectorization
      allocatable :: IDDM(:)
c--------------------------------------------------
c  if   i=nbrall(1,np),j=nbrall(2,np),nn=nbrall(3,np), then
c   nn-th bond is (i,j)pair.
c     nbrmax:  No. of (i,j) pairs
c  npr=nbrp(jj,i) shows bond number(npr) for jj-th adiacent atom
c  for i-th atom. This array is used with nbr(*,*).
c--------------------------------------------------
c neighbors' information
      allocatable ::  nbrall(:,:),nbrp(:,:)
      allocatable ::  nbr(:,:),nc(:)
c neighbors boundary condition
      allocatable ::  npbx(:,:),npby(:,:),npbz(:,:) 
c kind of atom ( 1:Si , 2:H , 3:C )
      allocatable ::  nkind(:)
      allocatable ::  ndivx(:),ndivy(:),ndivz(:)
c
c mapping function
c       allocatable :: map(:),imap(:)
c for multi-step 
c*      allocatable :: mlt(:),mltf(:),mltf2(:),imltf(:),imltf2(:)
c*      allocatable :: ntmp(:)
c*      allocatable :: nbrall_m(:,:),nbrp_m(:,:)
c*      allocatable :: nbr_m(:,:),nc_m(:)
c*      allocatable :: npbx_m(:,:),npby_m(:,:),npbz_m(:,:)
c*      allocatable :: nkind_m(:)
c*      allocatable :: ddx2_m(:),ddy2_m(:),ddz2_m(:)
c*      allocatable :: x_m(:),y_m(:),z_m(:)
c*      allocatable :: vx_m(:),vy_m(:),vz_m(:)
c*      allocatable :: xi_m(:),yi_m(:),zi_m(:)
c*      allocatable :: vxi_m(:),vyi_m(:),vzi_m(:)
c*      allocatable :: fx_m(:),fy_m(:),fz_m(:)
c*      allocatable :: dx_m(:),dy_m(:),dz_m(:)
c*      allocatable :: rtest_m(:)
c
c allocate - allocate - allocate - allocate - allocate 
c       iall5 = iall / nmlt
c-------------------------------------------------------------------
c mapping function 
c      allocate(map(iall),imap(iall))
c for multi-step
c*      allocate(mlt(iall),mltf(iall*4),mltf2(iall),imltf(iall)
c*     &,imltf2(iall))
c*      allocate(ntmp(iall*4))
c*      allocate(nbrall_m(4,iall5*ncut),nbrp_m(ncut,iall5))
c*      allocate(nbr_m(ncut,iall5),nc_m(iall5))
c*      allocate(npbx_m(ncut,iall5),npby_m(ncut,iall5)
c*     &,npbz_m(ncut,iall5))
c*      allocate(nkind_m(iall5))
c*      allocate(ddx2_m(iall5),ddy2_m(iall5),ddz2_m(iall5))
c*      allocate(x_m(iall5),y_m(iall5),z_m(iall5))
c*      allocate(vx_m(iall5),vy_m(iall5),vz_m(iall5))
c*      allocate(xi_m(iall5),yi_m(iall5),zi_m(iall5))
c*      allocate(vxi_m(iall5),vyi_m(iall5),vzi_m(iall5))
c*      allocate(fx_m(iall5),fy_m(iall5),fz_m(iall5))
c*      allocate(dx_m(iall5),dy_m(iall5),dz_m(iall5))
c*      allocate(rtest_m(iall5*ncut))
c
c       common /ndm/ nbrmax_m,mltfn
c
      end module work_arrays
c---------------------------------------------------------------------
      module connect
      implicit integer (i-n)
      implicit double precision (a-h,o-z)
      allocatable :: ux(:),uy(:),uz(:) ! atomic displacement
      allocatable :: x0(:),y0(:),z0(:) ! atomic initial position
      allocatable :: atomep(:,:) ! atomic level strain
cccc region II,III information
      allocatable :: pcor(:,:)
      allocatable :: ncor2(:,:),ncor(:,:)
      data d_con/0.4586913d0/  ! for internal displacement
      common /nr0/ nr2,nr3,ntdis0
c
      end module connect
c------------------------------------------------------------
c   MAIN PROGRAM   FORTRAN90 by S.izumi 2000-05-08
c------------------------------------------------------------
      program main
c      use fem_arrays
c      use connect
c      use work_arrays
      use fem_arrays
      use connect
c      use work_arrays
      implicit integer (i-n)
      implicit double precision (a-h,o-z)
      integer index
      integer*4 k
      character*60 arg
      external getarg
      k=1
      call getarg ( k, arg )
c
      index = lnblnk(arg)
      if (index.eq.0) then
         arg = 'files'
         index = lnblnk(arg)
c      stop 'need file-directory'
      endif
c     
      do i = 1,nfile
      fnum(i)(1:70) = arg(1:index)
      enddo
c
      fnum(1)(index+1:70) = '/datacheck.dat'
      fnum(2)(index+1:70) = '/result.dat'
      fnum(3)(index+1:70) = '/input.dat'
      fnum(4)(index+1:70) = '/ncor2.dat'
      fnum(5)(index+1:70) = '/ncor3.dat'
      fnum(6)(index+1:70) = '/cg_step.dat'
      fnum(7)(index+1:70) = '/cg669.dat'
      fnum(8)(index+1:70) = '/cg670.dat'
c
      open(77,file=fnum(1),status='unknown')
      open(666,file=fnum(6),status='unknown')
      open(669,file=fnum(7),status='unknown')
      open(670,file=fnum(8),status='unknown')
c   
      CALL DATA
cccc for fem-md start
      CALL DATA_NCOR ! read from ncor2.dat & ncor3.dat
      CALL MD_INIT   ! initilization of md calc.
      CALL DATA_MD   ! set additional displacement constrain 
cccc for fem-md end
c
      ek=0.0d0;f=0.0d0;gk=0.0d0  !   CALL CLEAR
c
      NA=0 ! NA :number of integration point
ccc For all element
      DO 150 IE=1,NTE
         CALL GAUSS ! gauss integration point
         CALL DMAT ! elastic matrix 
ccc For all integration point
        DO 100 K=1,NGNG(IE)
         XXI  = AXI(K);ETA  = AETA(K);ZETA = AZETA(K); HNUM = H(K)
         NA=NA+1
         CALL INT  ! infinite element
c      write(99,777)ie,k,XXI,ETA,ZETA,HNUM,N(1)
c      write(99,777)ie,k,DNDXI(1),DNDETA(1),DNDZETA(1),HNUM,N(1)
c 777  format(2i3,5e12.3)
c         write(6,*)IE,K
         CALL JMAT   ! infinite element
         CALL BMAT   ! B matrix
         CALL BMMAT  ! B matrix for each integration point (NA)
         CALL BDBMAT ! BtDB matrix
         CALL GAUSS2 ! element stiffness matrix 
 100     CONTINUE
c
         CALL MERGE ! element stiffness --> global stiffness
         EK=0.0d0      !  clear element stiffness matrix 
c         write(6,*)ie
 150  CONTINUE
      write(6,*)'MERGE END'
      CALL BOUND  !   boundary condition
      write(6,*)'BOUND END'
c  CALL SOLVE(GK,F,U) ! gauss-erase-method
c  CALL SOLVE_C(GK,GKC,F,U,DR,ZM) ! cholesky method
c  CALL SOLVE_CD(GKC,F,U,DR,ZM) ! cholesky method(displacement)
      CALL SOLVE_C !  ICCG  method
      write(6,*)'SLOVE_C END'
ccccc re-calculation of global stiffness matrix
      ek = 0.0d0 ;gk = 0.0d0 ! clear_r
c
      NA=0 ! NA :number of integration point
ccc For all element
      DO 155 IE=1,NTE
         CALL GAUSS ! gauss integration point
         CALL DMAT ! elastic matrix 
ccc For all integration point
        DO 105 K=1,NGNG(IE)
         XXI = AXI(K);ETA  = AETA(K);ZETA = AZETA(K); HNUM = H(K)
         NA=NA+1
         CALL INT  ! infinite element
c      write(99,777)ie,k,XI,ETA,ZETA,HNUM,N(1)
c      write(99,777)ie,k,DNDXI(1),DNDETA(1),DNDZETA(1),HNUM,N(1)
c         write(6,*)IE,K
         CALL JMAT   ! infinite element
         CALL BMAT   ! B matrix
         CALL BMMAT  ! B matrix for each integration point (NA)
         CALL BDBMAT ! BtDB matrix
         CALL GAUSS2 ! element stiffness matrix 
 105    CONTINUE
c
         CALL MERGE ! element stiffness --> global stiffness
         EK=0.0d0      !  clear element stiffness matrix 
c         write(6,*)ie
 155  CONTINUE
c
c
      CALL SOLVE_CG ! ICCG  method
      write(6,*)'SLOVE_CG END'
c
c      EP=0.0d0; SG=0.0d0    !  CLEAR
c      CALL STRAIN  ! strain B*a
c      CALL STRESS  ! stress D*strain
c      CALL OUTPUT_AV ! output(averange at node)
      open(3,FILE=fnum(2),status='unknown')
      CALL OUTPUT ! output
      CLOSE(3)
      call output_md
c
      STOP
      END
